/* ================================================================================================================================
  This script was created by SkyTruth to create annual greenest-pixel (maximum NDVI) composites and to save those composites as an 
  Earth Engine ImageCollection. This collection is used as input data for the annual coal surface mining extent analysis.
  This code differs from others written after the publication of the Plos paper because it preserves all band data and creates 
  multiple indexes.
-----------------------------------------------------------------------------------------------------------------------------------
	Bands included in the final Export:
	B, G, R, NIR, SWIR1, SWIR2, TIRS (from L4 &5), LG_TIRS (from L7), HG_TIRS (from L7), TIRS_1 (from L8), TIRS_2 (from L8)

	Indices created for the final Export:
	NDVI, NDMI, EVI, SAVI, MSAVI, NBR, NBR
-----------------------------------------------------------------------------------------------------------------------------------
	The methodology of the MTM data produced for PLOS One relied on the now deprecated Pre-Collection Landsat data. The USGS 
	Switched to the Collection format around May 2017. This change has prompted a shift to use USGS Landsat Collection 1 Tier 1 and 
	Real-Time Raw Scenes, and is reflected here. Read about the original study here: 
	https://journals.plos.org/plosone/article?id=10.1371/journal.pone.0197758
================================================================================================================================ */

// Import the Study Area
var studyArea = ee.FeatureCollection('users/skytruth-data/Plos_MTM_Fusion_Table_Backup/plosScriptFusionTableBackup/studyArea');
var exportbounds = studyArea.geometry().bounds();

// Import Image Collections
var lt4_raw = ee.ImageCollection('LANDSAT/LT04/C01/T1').select('B1','B2','B3','B4','B5','B6','B7');
var lt5_raw = ee.ImageCollection('LANDSAT/LT05/C01/T1').select('B1','B2','B3','B4','B5','B6','B7');
var le7_raw = ee.ImageCollection('LANDSAT/LE07/C01/T1').select('B1','B2','B3','B4','B5','B6_VCID_1','B6_VCID_2','B7','B8');
var lc8_raw = ee.ImageCollection('LANDSAT/LC08/C01/T1').select('B1','B2','B3','B4','B5','B6','B7','B8','B9','B10','B11');

var cleaner = function(outlierValue) {
  return function(image){
      var min_mask = image.mask().reduce(ee.Reducer.min());
      var sat_mask = image.reduce(ee.Reducer.max()).lt(outlierValue);
      var new_mask = min_mask.min(sat_mask).focal_min(3);
      return ee.Algorithms.Landsat.TOA(image).updateMask(new_mask);
  };
};

var maskClouds = function(image) {
  var scored = ee.Algorithms.Landsat.simpleCloudScore(image);
  return image.updateMask(scored.select(['cloud']).lt(20));
};

// Create a mask image which will be used to create matching thermal bangs
var blank = ee.Image().mask();
var newMaskBand = ee.Image().updateMask(blank).toFloat();

var lt4Collection = lt4_raw
  .filterBounds(studyArea)
  .filterDate('1984-01-01', '1993-12-31')
  .map(cleaner(250))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B3'),NIR:image.select('B4')}).rename('NDVI'))
    
    // NORMALIZED DIFFERENCE MOISTURE INDEX -- NDMI
    .addBands(image.expression('(NIR - SWIR1) / (NIR + SWIR1)',
    {NIR: image.select('B4'),SWIR1: image.select('B5')}).rename('NDMI'))
    
    // ENHANCED VEGETATION INDEX -- EVI
    .addBands(image.expression('(G * ((NIR - R) / (NIR + C1 * R - C2 * B + L)))',
    {G: 2.5,NIR: image.select('B4'),R: image.select('B3'),C1: 6,C2: 7.5,B: image.select('B1'),L: 1}).rename('EVI'))
    
    // SOIL ADJUSTED VEGETATION INDEX -- SAVI
    .addBands(image.expression('((C + L) * ((NIR - R) / (NIR + R + L)))',
    {NIR: image.select('B4'),R: image.select('B3'),  C: 1,L: 0.5}).rename('SAVI'))
    
    // MODIFIED SOIL ADJUSTED VEGETATION INDEX -- MSAVI
    .addBands(image.expression('((C * NIR - sqrt(((C * NIR + C2)**C3) - C4 * (NIR - R))) / C5)',
    {C: 2,NIR: image.select('B4'),R: image.select('B3'),  C2: 1,C3: 2,C4: 8,C5: 2}).rename('MSAVI'))
    
    // NORMALIZED BURN RATIO -- NBR
    .addBands(image.expression('(NIR - SWIR2) / (NIR + SWIR2)',
    {NIR: image.select('B4'),SWIR2: image.select('B7')}).rename('NBR'))
    
    // NORMALIZED BURN RATIO 2 -- NBR2
    .addBands(image.expression('(SWIR1 - SWIR2) / (SWIR1 + SWIR2)',
    {SWIR1: image.select('B5'),SWIR2: image.select('B7') }).rename('NBR2'))
    
    // LANDSAT 7 THERMAL BAND 1 -- LG_TIRS
    .addBands(newMaskBand.toFloat().rename('LG_TIRS'))
    
    // LANDSAT 7 THERMAL BAND 2 -- HG_TIRS
    .addBands(newMaskBand.toFloat().rename('HG_TIRS'))
    
    // LANDSAT 8 THERMAL BAND 1 -- TIRS_1
    .addBands(newMaskBand.toFloat().rename('TIRS_1'))
    
    // LANDSAT 8 THERMAL BAND 2 -- TIRS_2
    .addBands(newMaskBand.toFloat().rename('TIRS_2'))
    
  .select(['B1','B2','B3','B4','B5','B7','B6','LG_TIRS','HG_TIRS','TIRS_1','TIRS_2','NDVI','NDMI','EVI','SAVI','MSAVI','NBR','NBR2'])
  .rename(['B','G','R','NIR','SWIR1','SWIR2','TIRS','LG_TIRS','HG_TIRS','TIRS_1','TIRS_2','NDVI','NDMI','EVI','SAVI','MSAVI','NBR','NBR2']);
  });

var lt5Collection = lt5_raw
  .filterBounds(studyArea)
  .filterDate('1984-01-01', '2011-12-31')
  .map(cleaner(250))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B3'),NIR:image.select('B4')}).rename('NDVI'))
    
    // NORMALIZED DIFFERENCE MOISTURE INDEX -- NDMI
    .addBands(image.expression('(NIR - SWIR1) / (NIR + SWIR1)',
    {NIR: image.select('B4'),SWIR1: image.select('B5')}).rename('NDMI'))
    
    // ENHANCED VEGETATION INDEX -- EVI
    .addBands(image.expression('(G * ((NIR - R) / (NIR + C1 * R - C2 * B + L)))',
    {G: 2.5,NIR: image.select('B4'),R: image.select('B3'),C1: 6,C2: 7.5,B: image.select('B1'),L: 1}).rename('EVI'))
    
    // SOIL ADJUSTED VEGETATION INDEX -- SAVI
    .addBands(image.expression('((C + L) * ((NIR - R) / (NIR + R + L)))',
    {NIR: image.select('B4'),R: image.select('B3'),  C: 1,L: 0.5}).rename('SAVI'))
    
    // MODIFIED SOIL ADJUSTED VEGETATION INDEX -- MSAVI
    .addBands(image.expression('((C * NIR - sqrt(((C * NIR + C2)**C3) - C4 * (NIR - R))) / C5)',
    {C: 2,NIR: image.select('B4'),R: image.select('B3'),  C2: 1,C3: 2,C4: 8,C5: 2}).rename('MSAVI'))
    
    // NORMALIZED BURN RATIO -- NBR
    .addBands(image.expression('(NIR - SWIR2) / (NIR + SWIR2)',
    {NIR: image.select('B4'),SWIR2: image.select('B7')}).rename('NBR'))
    
    // NORMALIZED BURN RATIO 2 -- NBR2
    .addBands(image.expression('(SWIR1 - SWIR2) / (SWIR1 + SWIR2)',
    {SWIR1: image.select('B5'),SWIR2: image.select('B7') }).rename('NBR2'))
    
    // LANDSAT 7 THERMAL BAND 1 -- LG_TIRS
    .addBands(newMaskBand.toFloat().rename('LG_TIRS'))
    
    // LANDSAT 7 THERMAL BAND 2 -- HG_TIRS
    .addBands(newMaskBand.toFloat().rename('HG_TIRS'))
    
    // LANDSAT 8 THERMAL BAND 1 -- TIRS_1
    .addBands(newMaskBand.toFloat().rename('TIRS_1'))
    
    // LANDSAT 8 THERMAL BAND 2 -- TIRS_2
    .addBands(newMaskBand.toFloat().rename('TIRS_2'))
    
  .select(['B1','B2','B3','B4','B5','B7','B6','LG_TIRS','HG_TIRS','TIRS_1','TIRS_2','NDVI','NDMI','EVI','SAVI','MSAVI','NBR','NBR2'])
  .rename(['B','G','R','NIR','SWIR1','SWIR2','TIRS','LG_TIRS','HG_TIRS','TIRS_1','TIRS_2','NDVI','NDMI','EVI','SAVI','MSAVI','NBR','NBR2']);
  });

var le7Collection = le7_raw
  .filterBounds(studyArea)
  .filterDate('1999-01-01', Date.now())
  .filter(ee.Filter.date("2012-08-03","2012-08-04").not()) // This line removes a specific image that has too many errors
  .map(cleaner(250))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B3'),NIR:image.select('B4')}).rename('NDVI'))
    
    // NORMALIZED DIFFERENCE MOISTURE INDEX -- NDMI
    .addBands(image.expression('(NIR - SWIR1) / (NIR + SWIR1)',
    {NIR: image.select('B4'),SWIR1: image.select('B5')}).rename('NDMI'))
    
    // ENHANCED VEGETATION INDEX -- EVI
    .addBands(image.expression('(G * ((NIR - R) / (NIR + C1 * R - C2 * B + L)))',
    {G: 2.5,NIR: image.select('B4'),R: image.select('B3'),C1: 6,C2: 7.5,B: image.select('B1'),L: 1}).rename('EVI'))
    
    // SOIL ADJUSTED VEGETATION INDEX -- SAVI
    .addBands(image.expression('((C + L) * ((NIR - R) / (NIR + R + L)))',
    {NIR: image.select('B4'),R: image.select('B3'),  C: 1,L: 0.5}).rename('SAVI'))
    
    // MODIFIED SOIL ADJUSTED VEGETATION INDEX -- MSAVI
    .addBands(image.expression('((C * NIR - sqrt(((C * NIR + C2)**C3) - C4 * (NIR - R))) / C5)',
    {C: 2,NIR: image.select('B4'),R: image.select('B3'),  C2: 1,C3: 2,C4: 8,C5: 2}).rename('MSAVI'))
    
    // NORMALIZED BURN RATIO -- NBR
    .addBands(image.expression('(NIR - SWIR2) / (NIR + SWIR2)',
    {NIR: image.select('B4'),SWIR2: image.select('B7')}).rename('NBR'))
    
    // NORMALIZED BURN RATIO 2 -- NBR2
    .addBands(image.expression('(SWIR1 - SWIR2) / (SWIR1 + SWIR2)',
    {SWIR1: image.select('B5'),SWIR2: image.select('B7') }).rename('NBR2'))
    
    // LANDSAT 4-5 THERMAL BAND -- TIRS
    .addBands(newMaskBand.rename('TIRS'))
    
    // LANDSAT 8 THERMAL BAND 1 -- TIRS_1
    .addBands(newMaskBand.rename('TIRS_1'))
    
    // LANDSAT 8 THERMAL BAND 2 -- TIRS_2
    .addBands(newMaskBand.rename('TIRS_2'))
    
  .select(['B1','B2','B3','B4','B5','B7','TIRS','B6_VCID_1','B6_VCID_2','TIRS_1','TIRS_2','NDVI','NDMI','EVI','SAVI','MSAVI','NBR','NBR2'])
  .rename(['B','G','R','NIR','SWIR1','SWIR2','TIRS','LG_TIRS','HG_TIRS','TIRS_1','TIRS_2','NDVI','NDMI','EVI','SAVI','MSAVI','NBR','NBR2']);
  });

var lc8Collection = lc8_raw
  .filterBounds(studyArea)
  .filterDate('2013-01-01', Date.now())
  .map(cleaner(64256))
  .map(maskClouds)
  .map(function(image) {
    return image
    // NORMALIZED DIFFERENCE VEGETATION INDEX -- NDVI
    .addBands(image.expression('(NIR - R) / (NIR + R)',
    {R:image.select('B4'),NIR:image.select('B5')}).rename('NDVI'))
    
    // NORMALIZED DIFFERENCE MOISTURE INDEX -- NDMI
    .addBands(image.expression('(NIR - SWIR1) / (NIR + SWIR1)',
    {NIR: image.select('B5'),SWIR1: image.select('B6')}).rename('NDMI'))
    
    // ENHANCED VEGETATION INDEX -- EVI
    .addBands(image.expression('(G * ((NIR - R) / (NIR + C1 * R - C2 * B + L)))',
    {G: 2.5,NIR: image.select('B5'),R: image.select('B4'),C1: 6,C2: 7.5,B: image.select('B2'),L: 1}).rename('EVI'))
    
    // SOIL ADJUSTED VEGETATION INDEX -- SAVI
    .addBands(image.expression('((C + L) * ((NIR - R) / (NIR + R + L)))',
    {NIR: image.select('B5'),R: image.select('B4'),  C: 1,L: 0.5}).rename('SAVI'))
    
    // MODIFIED SOIL ADJUSTED VEGETATION INDEX -- MSAVI
    .addBands(image.expression('((C * NIR - sqrt(((C * NIR + C2)**C3) - C4 * (NIR - R))) / C5)',
    {C: 2,NIR: image.select('B5'),R: image.select('B4'),  C2: 1,C3: 2,C4: 8,C5: 2}).rename('MSAVI'))
    
    // NORMALIZED BURN RATIO -- NBR
    .addBands(image.expression('(NIR - SWIR2) / (NIR + SWIR2)',
    {NIR: image.select('B5'),SWIR2: image.select('B7')}).rename('NBR'))
    
    // NORMALIZED BURN RATIO 2 -- NBR2
    .addBands(image.expression('(SWIR1 - SWIR2) / (SWIR1 + SWIR2)',
    {SWIR1: image.select('B6'),SWIR2: image.select('B7') }).rename('NBR2'))
    
    // LANDSAT 4-5 THERMAL BAND -- TIRS
    .addBands(newMaskBand.rename('TIRS'))
    
    // LANDSAT 7 THERMAL BAND 1 -- LG_TIRS
    .addBands(newMaskBand.rename('LG_TIRS'))
    
    // LANDSAT 7 THERMAL BAND 2 -- HG_TIRS
    .addBands(newMaskBand.rename('HG_TIRS'))
    
  .select(['B2','B3','B4','B5','B6','B7','TIRS','LG_TIRS','HG_TIRS','B10','B11','NDVI','NDMI','EVI','SAVI','MSAVI','NBR','NBR2'])
  .rename(['B','G','R','NIR','SWIR1','SWIR2','TIRS','LG_TIRS','HG_TIRS','TIRS_1','TIRS_2','NDVI','NDMI','EVI','SAVI','MSAVI','NBR','NBR2']);
  });

var lsCollection = ee.ImageCollection((lt4Collection)
    .merge(lt5Collection)
    .merge(le7Collection)
    .merge(lc8Collection));

/*------------------------ EXPORTING YEARLY COMPOSITES -----------------------*/
var year = 1984; // 1984-2019

// Make the annual composite for the year noted above
var composite = lsCollection
      .filter(ee.Filter.calendarRange(year, year, 'year'))
      
      .filter(ee.Filter.calendarRange(5, 9, 'month'))
        // The above line is for restricting imagery to summer months. Using the
        // same date range as that defined in PLOS One
      .qualityMosaic('NDVI')
      .set({"year": year}); // This adds the year as a property to the image

// Export to GEE asset
var description = "composite_"+year; // Be careful here with multi-year MSS
Map.addLayer(composite,{bands:['R','G','B'],min:0,max:0.3}, year+' Composite',false);

Export.image.toAsset({
  image: composite,
  description: description,
  assetId: ("users/skytruth-data/MTR/greenestPixelComposites_allBand_allIndex/"+description),
  region: exportbounds,
  scale: 30,
  maxPixels: 1e10
});
